import type {
  GroupListResponseModel,
  ListViewConfig,
  SchoolClassDetailResponseModel,
  WorkGroupDetailResponseModel,
} from '@/helpers/models';
import {fetchAuthenticated} from '@/helpers/retrieveData';
import {useErrorStore} from '@/stores/error';
import i18next from 'i18next';
import type {UGridRow} from '@univention/univention-veb';
import {SubElement} from '@univention/univention-veb';


export const getError = async(response: Response): Promise<{detail: unknown} | null> => {
  if (!response.ok) {
    const text = await response.text();
    let detail = {
      detail: text,
    };
    try {
      const json = JSON.parse(text);
      if (json && typeof json === 'object' && 'detail' in json) {
        detail = json;
      }
    } catch (err) {
      // Either not a JSON response, or unparsable. We'll just ignore this.
    }
    return detail;
  }
  return null;
};

export type SaveError = {
  type: 'generic';
  message: string;
} | {
  type: 'fieldErrors';
  errors: {
    field: string;
    message: string;
  }[];
};
export const getSaveError = async(response: Response, context: 'updateWorkgroup' | 'createWorkgroup' | 'updateSchoolClass'): Promise<SaveError | null> => {
  const errorJson = await getError(response);
  if (!errorJson) {
    return null;
  }
  if (response.status === 422) {
    // FIXME validate against json.schema
    if (Array.isArray(errorJson.detail)) {
      const detail: {loc: string[]; msg: string; type: string}[] = errorJson.detail;
      const error: SaveError = {
        type: 'fieldErrors',
        errors: [],
      };
      for (const iDetail of detail) {
        const field = iDetail.loc[iDetail.loc.length - 1];
        if (field) {
          error.errors.push({
            field,
            message: iDetail.msg,
          });
        }
      }
      return error;
    }
  }
  if (response.status === 404) {
    return {
      type: 'generic',
      message: i18next.t(`dataAccess.${context}.error.404`),
    };
  }
  return {
    type: 'generic',
    message: String(errorJson.detail),
  };
};

const fetchListViewConfigMocked = (): ListViewConfig => {
  const _config: ListViewConfig = {
    url: 'url',
    allowedGlobalActions: ['add'],
    searchFormOptions: {
      schools: [
        {
          value: 'DEMOSCHOOL',
          label: 'DEMOSCHOOL',
        }, {
          value: 'School1',
          label: 'Schule 1',
        },
      ],
      divisClasstype: [
        {
          value: 'class',
          label: 'Class',
        },
        {
          value: 'offer',
          label: 'Offer',
        },
      ],
      activationStates: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
      isLmsGroup: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
      isIservGroup: [
        {
          value: 'disabled',
          label: 'Disabled',
        },
        {
          value: 'enabled',
          label: 'Enabled',
        },
      ],
    },
    columns: [
      {
        label: 'School year',
        attribute: 'idiSchoolyear',
      },
      {
        label: 'Name',
        attribute: 'name',
      },
      {
        label: 'Description',
        attribute: 'description',
      },
      {
        label: 'Group type',
        attribute: 'divisClasstype',
      },
      {
        label: 'Source UID',
        attribute: 'groupSourceId',
      },
      {
        label: 'Creation date',
        attribute: 'createTimestamp',
        formatter: 'date',
      },
      {
        label: 'Date of last change',
        attribute: 'modifyTimestamp',
        formatter: 'date',
      },
      {
        label: 'Record UID',
        attribute: 'recordUID',
      },
      {
        label: 'Activation state',
        attribute: 'disabled',
        formatter: 'boolean:Inactive:Active',
      },
      {
        label: 'Activation date',
        attribute: 'divisStartdate',
        formatter: 'date',
      },
      {
        label: 'Expiry date',
        attribute: 'divisEnddate',
        formatter: 'date',
      },
      {
        label: 'Scheduled deletion date',
        attribute: 'purgeDate',
        formatter: 'date',
      },
      {
        label: 'LMS enabled',
        attribute: 'isLmsGroup',
        formatter: 'boolean:LMS enabled:LMS disabled',
      },
      {
        label: 'IServ enabled',
        attribute: 'isIservGroup',
        formatter: 'boolean:IServ enabled:IServ disabled',
      },
    ],
  };
  console.debug('mock config', _config);
  return _config;
};


export const fetchListViewConfig = async(): Promise<ListViewConfig> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchListViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchListViewConfigMocked());
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated('/ucsschool/bff-groups/v1/pageconf/listView');
  const error = await getError(answer);
  if (error) {
    const errorStore = useErrorStore();
    const title = i18next.t('dataAccess.fetchListViewConfig.error.title');
    errorStore.push({
      title,
      message: JSON.stringify(error.detail),
      unRecoverable: true,
    });
    return Promise.reject(answer);
  }
  return await answer.json();
};


const fetchGroupsMocked = (): GroupListResponseModel[] => {
  /* eslint-disable camelcase */
  const group: GroupListResponseModel = {
    id: 'school1-group1',
    allowedActions: ['edit', 'delete'],
    schoolGroupType: 'workgroup',
    attributes: {
      idiSchoolyear: {
        value: 'Year 3',
        access: 'read',
      },
      name: {
        value: 'school1-group1',
        access: 'read',
      },
      description: {
        value: 'Group 1',
        access: 'read',
      },
      divisClasstype: {
        value: 'Group type A',
        access: 'read',
      },
      groupSourceId: {
        value: 'sourceID',
        access: 'read',
      },
      createTimestamp: {
        value: '2020-10-5',
        access: 'read',
      },
      modifyTimestamp: {
        value: '2020-10-05',
        access: 'read',
      },
      recordUID: {
        value: 'recordUID',
        access: 'read',
      },
      disabled: {
        value: false,
        access: 'read',
      },
      divisStartdate: {
        value: '2020-10-05',
        access: 'read',
      },
      divisEnddate: {
        value: '2020-10-05',
        access: 'read',
      },
      purgeDate: {
        value: '2020-10-05',
        access: 'read',
      },
      isLmsGroup: {
        value: true,
        access: 'read',
      },
      isIservGroup: {
        value: false,
        access: 'read',
      },
    },
  };
  /* eslint-enable camelcase */

  const _groups: GroupListResponseModel[] = [group];

  for (let x = 2; x < 500; x++) {
    const y = JSON.parse(JSON.stringify(group));
    y.id = `school1-group${x}`;
    y.attributes.name.value = y.id;
    if (x % 2 === 0) {
      y.allowedActions = ['edit'];
    }
    y.schoolGroupType = 'school_class';
    _groups.push(y);
  }
  console.log('mock groups: ', _groups);
  return _groups;
};

export const fetchGroups = async(
  school: string,
  schoolYear: string,
  divisClassType: string,
  source: string,
  activationState: string,
  isLmsGroup: string,
  iServEnabled: string,
  quickSearch: string,
  limit?: number,
): Promise<GroupListResponseModel[] | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchGroups. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchGroupsMocked());
      }, 1000);
    });
  }

  const urlSearchParams = new URLSearchParams({school});
  if (schoolYear !== '') {
    urlSearchParams.append('schoolYear', schoolYear);
  }
  if (divisClassType !== '') {
    urlSearchParams.append('divisClassType', divisClassType);
  }
  if (source !== '') {
    urlSearchParams.append('source', source);
  }
  if (activationState !== '') {
    urlSearchParams.append('activationState', activationState);
  }
  if (isLmsGroup !== '') {
    urlSearchParams.append('isLmsGroup', (isLmsGroup === 'enabled').toString());
  }
  if (iServEnabled !== '') {
    urlSearchParams.append('isIservGroup', (iServEnabled === 'enabled').toString());
  }
  if (quickSearch !== '') {
    urlSearchParams.append('quickSearch', quickSearch);
  }
  if (limit) {
    urlSearchParams.append('limit', `${limit}`);
  }

  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/groups?${urlSearchParams.toString()}`);
  const error = await getError(answer);
  if (error) {
    if (typeof error.detail === 'object' && error.detail !== null && 'type' in error.detail) {
      const detail = error.detail as {type: string};
      if (detail.type === 'SearchLimitReached') {
        return null;
      }
    }
    const errorStore = useErrorStore();
    const title = i18next.t('dataAccess.fetchGroups.error.title');
    errorStore.push({
      title,
      message: JSON.stringify(error.detail),
    });
    return Promise.reject(answer);
  }
  return await answer.json();
};


export const deleteGroups = async(groups: GroupListResponseModel[]): Promise<{id: string; error: string}[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('deleteGroups. VITE_USE_REAL_BACKEND===false in DEV mode. Doing nothing');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve([
          {
            id: groups[0]?.id as string,
            error: 'some error',
          },
        ]);
      }, 1000);
    });
  }

  const responses = await Promise.all(groups.map(group => fetchAuthenticated(`/ucsschool/bff-groups/v1/${group.schoolGroupType}/${group.id}`, {
    method: 'DELETE',
  })));
  const errors = await Promise.all(responses.map(x => getError(x)));
  const fails: {id: string; error: string}[] = [];
  for (let x = 0; x < responses.length; x++) {
    const error = errors[x];
    if (error) {
      fails.push({
        id: groups[x]?.id as string,
        error: JSON.stringify(error.detail),
      });
    }
  }
  return fails;
};

const fetchMembersMocked = (): UGridRow[] => [
  {
    id: 'max.mustermann',
    attributes: {
      username: {
        value: 'max.mustermann',
        access: 'read',
      },
      firstname: {
        value: 'Max',
        access: 'read',
      },
      lastname: {
        value: 'Mustermann',
        access: 'read',
      },
      role: {
        value: 'student',
        access: 'read',
      },
    },
  },
  {
    id: 'hans.peter',
    attributes: {
      username: {
        value: 'hans.peter',
        access: 'read',
      },
      firstname: {
        value: 'Hans',
        access: 'read',
      },
      lastname: {
        value: 'Peter',
        access: 'read',
      },
      role: {
        value: 'student',
        access: 'read',
      },
    },
  },
  {
    id: 'walter.scholl',
    attributes: {
      username: {
        value: 'walter.scholl',
        access: 'read',
      },
      firstname: {
        value: 'Walter',
        access: 'read',
      },
      lastname: {
        value: 'Scholl',
        access: 'read',
      },
      role: {
        value: 'student',
        access: 'read',
      },
    },
  },
];

export const fetchMembers = async(school: string, values: Record<string, unknown>): Promise<UGridRow[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchMembers. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchMembersMocked());
      }, 1000);
    });
  }

  const urlSearchParams = new URLSearchParams({school});
  for (const key in values) {
    const value = values[key];
    if (typeof value === 'string' && value !== '') {
      urlSearchParams.append(key, value);
    }
  }
  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/users?${urlSearchParams.toString()}`);
  const error = await getError(answer);
  if (error) {
    const errorStore = useErrorStore();
    const title = i18next.t('dataAccess.fetchMembers.error.title');
    errorStore.push({
      title,
      message: JSON.stringify(error.detail),
    });
    return Promise.reject(answer);
  }
  return await answer.json();
};


export const getDetailResponseModelMocked = (): WorkGroupDetailResponseModel | SchoolClassDetailResponseModel => ({
  url: '',
  allowedActions: ['save'],
  pages: [
    {
      label: 'General',
      name: 'general',
      fieldsets: [
        {
          label: 'Basic settings',
          name: 'general__basic',
          rows: [
            [
              {
                type: 'UInputText',
                props: {
                  name: 'idiSchoolyear',
                  label: 'School year',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  name: 'description',
                  label: 'Description',
                  required: true,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputText',
                props: {
                  name: 'name',
                  label: 'Name',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UMultiObjectSelect',
                props: {
                  name: 'users',
                  label: 'Users',
                  access: 'write',
                  searchStandby: false,
                  initialRows: [
                    {
                      id: 'max.mustermann',
                      attributes: {
                        username: {
                          value: 'max.mustermann',
                          access: 'read',
                        },
                        firstname: {
                          value: 'Max',
                          access: 'read',
                        },
                        lastname: {
                          value: 'Mustermann',
                          access: 'read',
                        },
                        role: {
                          value: 'student',
                          access: 'read',
                        },
                      },
                    },
                  ],
                  columns: [
                    {
                      label: 'Username',
                      attribute: 'username',
                    },
                    {
                      label: 'Firstname',
                      attribute: 'firstname',
                    },
                    {
                      label: 'Lastname',
                      attribute: 'lastname',
                    },
                    {
                      label: 'Role',
                      attribute: 'role',
                    },
                  ],
                  searchRows: [],
                  searchElements: [
                    {
                      type: SubElement.UComboBox,
                      props: {
                        label: 'Role',
                        name: 'role',
                        options: [
                          {
                            value: 'role1',
                            label: 'Role 1',
                          },
                          {
                            value: 'role2',
                            label: 'Role 2',
                          },
                        ],
                      },
                    },
                    {
                      type: SubElement.UComboBox,
                      props: {
                        label: 'Schoolclass',
                        name: 'schoolClass',
                        options: [
                          {
                            value: 'class1',
                            label: 'Class 1',
                          },
                          {
                            value: 'class2',
                            label: 'Class 2',
                          },
                        ],
                      },
                    },
                    {
                      type: SubElement.UComboBox,
                      props: {
                        label: 'Workgroup',
                        name: 'workgroup',
                        options: [
                          {
                            value: 'workgroup1',
                            label: 'Workgroup 1',
                          },
                          {
                            value: 'workgroup2',
                            label: 'Workgroup 2',
                          },
                        ],
                      },
                    },
                    {
                      type: SubElement.UInputText,
                      props: {
                        label: 'Suche',
                        name: 'quickSearch',
                      },
                    },
                  ],
                  searchColumns: [
                    {
                      label: 'Username',
                      attribute: 'username',
                    },
                    {
                      label: 'Firstname',
                      attribute: 'firstname',
                    },
                    {
                      label: 'Lastname',
                      attribute: 'lastname',
                    },
                    {
                      label: 'Role',
                      attribute: 'role',
                    },
                  ],
                },
              },
            ],
            [
              {
                type: 'UComboBox',
                props: {
                  name: 'divisClasstype',
                  label: 'Group type',
                  required: false,
                  access: 'write',
                  options: [
                    {
                      label: 'Class',
                      value: 'class',
                    },
                    {
                      label: 'Offer',
                      value: 'offer',
                    },
                  ],
                },
              },
              {
                type: 'UComboBox',
                props: {
                  name: 'divisCoursetype',
                  label: 'Course type',
                  required: false,
                  access: 'write',
                  options: [
                    {
                      label: 'Course 1',
                      value: 'type1',
                    },
                    {
                      label: 'Course 2',
                      value: 'type2',
                    },
                  ],
                },
              },
            ],
            [
              {
                type: 'UInputText',
                props: {
                  label: 'Source',
                  name: 'groupSourceId',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  label: 'Record ID',
                  name: 'recordUID',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'LMS enabled',
                  name: 'isLmsGroup',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'IServ enabled',
                  name: 'isIservGroup',
                  required: false,
                  access: 'write',
                },
              },
            ],
          ],
        },
      ],
    },
    {
      label: 'Account',
      name: 'account',
      fieldsets: [
        {
          label: 'General',
          name: 'account__general',
          rows: [
            [
              {
                type: 'UInputDate',
                props: {
                  label: 'Creation date',
                  name: 'createTimestamp',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputDate',
                props: {
                  label: 'Date of last change',
                  name: 'modifyTimestamp',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputDate',
                props: {
                  label: 'Activation date',
                  name: 'divisStartdate',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputDate',
                props: {
                  label: 'Expiration date',
                  name: 'divisEnddate',
                  required: false,
                  access: 'write',
                },
              },
            ],
            [
              {
                type: 'UInputDate',
                props: {
                  label: 'Scheduled deletion date',
                  name: 'purgeDate',
                  required: false,
                  access: 'write',
                },
              },
            ],
          ],
        },
        {
          label: 'SAML settings',
          name: 'account__samlSettings',
          rows: [
            {
              type: 'UMultiSelect',
              props: {
                name: 'serviceprovidergroup',
                label: 'SAML service provider',
                required: false,
                columns: [
                  {
                    label: 'Select all',
                    attribute: 'label',
                  },
                ],
                rows: [
                  {
                    id: 'sp1',
                    attributes: {
                      label: {
                        value: 'Service provider 1',
                        access: 'read',
                      },
                    },
                  },
                  {
                    id: 'sp2',
                    attributes: {
                      label: {
                        value: 'Service provider 2',
                        access: 'read',
                      },
                    },
                  },
                  {
                    id: 'sp3',
                    attributes: {
                      label: {
                        value: 'Service provider 3',
                        access: 'read',
                      },
                    },
                  },
                ],
              },
            },
          ],
        },
      ],
    },
  ],
  values: {
    idiSchoolyear: '2020',
    description: 'description',
    name: 'username',
    users: ['max.mustermann'],
    divisClasstype: 'class',
    divisCoursetype: 'type1',
    groupSourceId: 'source',
    recordUID: 'record id',
    isLmsGroup: true,
    isIservGroup: true,
    createTimestamp: '2020-01-01',
    modifyTimestamp: '2020-01-01',
    divisStartdate: '2020-01-01',
    divisEnddate: '2020-01-01',
    purgeDate:  '2020-01-01',
    serviceprovidergroup: [],
  },
});
